==> wikitty-api-3.3-sources.jar: org/nuiton/wikitty/entities/FieldType.java <==
/*
 * #%L
 * Wikitty :: api
 * 
 * $Id: FieldType.java 1259 2011-12-28 18:50:45Z bpoussin $
 * $HeadURL: http://svn.nuiton.org/svn/wikitty/tags/wikitty-3.3/wikitty-api/src/main/java/org/nuiton/wikitty/entities/FieldType.java $
 * %%
 * Copyright (C) 2009 - 2010 CodeLutin, Benjamin Poussin
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 * 
 * You should have received a copy of the GNU General Lesser Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * #L%
 */
package org.nuiton.wikitty.entities;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;
import org.nuiton.wikitty.WikittyException;
import org.nuiton.wikitty.WikittyUtil;

/**
 *
 * @author poussin
 * @version $Revision: 1259 $
 *
 * Last update: $Date: 2011-12-28 19:50:45 +0100 (Wed, 28 Dec 2011) $
 * by : $Author: bpoussin $
 */
public class FieldType implements Serializable {

    /** serialVersionUID. */
    private static final long serialVersionUID = -4375308750387837026L;

    /** tag/value use for unique */
    static public String UNIQUE = "unique";
    /** tag/value use for not null */
    static public String NOT_NULL = "notNull";
    
    static public enum TYPE {
        BINARY, BOOLEAN, DATE, NUMERIC, STRING, WIKITTY;

        /**
         * convert string to TYPE, this method accept not trimed and not well
         * cased string (difference with valueOf)
         * @param name
         * @return TYPE else exception is thrown
         */
        public static TYPE parse(String name) {
            TYPE result = valueOf(name.trim().toUpperCase());
            return result;
        }
    }

    public static final int NOLIMIT = Integer.MAX_VALUE;

    protected TYPE type;
    protected int lowerBound;
    protected int upperBound;

    /** used to store tag/value used by client side ex: editor=xhtml */
    Map<String, String> tagValues = new HashMap<String, String>();

    public FieldType() {
    }

    public FieldType(TYPE type, int lowerBound, int upperBound) {
        this.type = type;
        this.lowerBound = lowerBound;
        this.upperBound = upperBound;
    }

    public void addTagValue(String tag, String value) {
        tagValues.put(tag, value);
    }

    public String getTagValue(String tag) {
        String result = tagValues.get(tag);
        return result;
    }

    public Set<String> getTagNames() {
        return tagValues.keySet();
    }

    public Map<String, String> getTagValues() {
        return tagValues;
    }

    public void setTagValues(Map<String, String> tagValues) {
        this.tagValues = tagValues;
    }

    /**
     * Return true if this field have upperBound > 1.
     * 
     * @return {@code true} is field is collection
     */
    public boolean isCollection() {
        return upperBound > 1;
    }

    /**
     * Return string definition for this field.
     * 
     * @param name field name used for definition
     * @return field definition
     */
    public String toDefinition(String name) {
        String result = type + " " + name;
        if (lowerBound != 0 || upperBound != 0) {
            if (upperBound != NOLIMIT) {
                result += "[" + lowerBound + "-" + upperBound + "]";
            } else {
                result += "[" + lowerBound + "-*]";
            }
        }
        result += WikittyUtil.tagValuesToString(tagValues);
        return result;
    }

    /**
     * Convert value in argument in right type for this FieldType. Don't support
     * collection.
     *
     * @param value value to convert
     * @return object in type of this FieldType
     */
    public Object getContainedValidObject( Object value ) {
        Object result = null;
        switch (type) {
        case BINARY:
            result = WikittyUtil.toBinary(value); break;
        case DATE:
            result = WikittyUtil.toDate(value); break;
        case NUMERIC:
            result = WikittyUtil.toBigDecimal(value); break;
        case BOOLEAN:
            result = WikittyUtil.toBoolean(value); break;
        case STRING:
            result = WikittyUtil.toString(value); break;
        default:
            // if type is not found then type is business type
            // and is wikitty object
            result = WikittyUtil.toWikitty(value); break;
        }
        return result;
    }

    /**
     * Return a valid value for this field.
     * 
     * @param value is casted if possible to an actual correct value.
     * @return value validity
     * @throws WikittyException if value can't be obtained
     */
    public Object getValidValue(Object value) throws WikittyException {
        if (value == null && isNotNull()) {
            throw new WikittyException("Value can't be null for this field");
        }

        Object result;
        if (value == null) {
            result = null;
        } else if (isCollection()) {
            if ( !(value instanceof Collection) ) {
                throw new WikittyException( "A collection is expected for type "
                        + type.name() + "[" + lowerBound + " - " + upperBound + "]" );
            }
            Collection<Object> col;
            if (isUnique()) {
                col = new LinkedHashSet<Object>();
            } else {
                col = new ArrayList<Object>();
            }

            // copy all value in new collections
            for ( Object o : (Collection<?>) value ) {
                col.add( getContainedValidObject(o) );
            }
            result = col;

            // check bound condition
            if (!(getLowerBound() <= col.size() && col.size() <= getUpperBound())) {
                // if upper bound reached, throw an exception
                throw new WikittyException(String.format(
                        "Can't set collection value, bad size collection: %s <= %s <= %s is false",
                        getLowerBound(), col.size(), getUpperBound()));
            }
        } else {
            result = getContainedValidObject(value);
        }
        return result;
    }

    /**
     * Test if value in argument is valid for this field type.
     * 
     * @param value to test
     * @return true if value is valid
     */
    public boolean isValidValue(Object value) {
        return getValidValue(value) != null;
    }

    public TYPE getType() {
        return type;
    }

    public void setType(TYPE type) {
        this.type = type;
    }

    public int getLowerBound() {
        return lowerBound;
    }

    public void setLowerBound(int lowerBound) {
        this.lowerBound = lowerBound;
    }

    public int getUpperBound() {
        return upperBound;
    }

    public void setUpperBound(int upperBound) {
        this.upperBound = upperBound;
    }

    public boolean isUnique() {
        return "true".equalsIgnoreCase(getTagValue(UNIQUE));
    }

    public boolean isNotNull() {
        return "true".equalsIgnoreCase(getTagValue(NOT_NULL));
    }

}

==> wikitty-api-3.4-sources.jar: org/nuiton/wikitty/entities/FieldType.java <==
/*
 * #%L
 * Wikitty :: api
 * 
 * $Id: FieldType.java 1322 2012-01-13 14:45:32Z bpoussin $
 * $HeadURL: http://svn.nuiton.org/svn/wikitty/tags/wikitty-3.4/wikitty-api/src/main/java/org/nuiton/wikitty/entities/FieldType.java $
 * %%
 * Copyright (C) 2009 - 2010 CodeLutin, Benjamin Poussin
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 * 
 * You should have received a copy of the GNU General Lesser Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * #L%
 */
package org.nuiton.wikitty.entities;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedHashSet;
import java.util.Map;
import java.util.Set;
import org.nuiton.wikitty.WikittyException;
import org.nuiton.wikitty.WikittyUtil;

/**
 *
 * @author poussin
 * @version $Revision: 1322 $
 *
 * Last update: $Date: 2012-01-13 15:45:32 +0100 (ven, 13 jan 2012) $
 * by : $Author: bpoussin $
 */
public class FieldType implements Serializable {

    /** serialVersionUID. */
    private static final long serialVersionUID = -4375308750387837026L;

    /** tag/value use for unique */
    static public String UNIQUE = "unique";
    /** tag/value use for not null */
    static public String NOT_NULL = "notNull";

    public static final int NOLIMIT = Integer.MAX_VALUE;

    protected WikittyTypes type;
    protected int lowerBound;
    protected int upperBound;

    /** used to store tag/value used by client side ex: editor=xhtml */
    Map<String, String> tagValues = new HashMap<String, String>();

    public FieldType() {
    }

    public FieldType(WikittyTypes type, int lowerBound, int upperBound) {
        this.type = type;
        this.lowerBound = lowerBound;
        this.upperBound = upperBound;
    }

    public void addTagValue(String tag, String value) {
        tagValues.put(tag, value);
    }

    public String getTagValue(String tag) {
        String result = tagValues.get(tag);
        return result;
    }

    public Set<String> getTagNames() {
        return tagValues.keySet();
    }

    public Map<String, String> getTagValues() {
        return tagValues;
    }

    public void setTagValues(Map<String, String> tagValues) {
        this.tagValues = tagValues;
    }

    /**
     * Return true if this field have upperBound > 1.
     * 
     * @return {@code true} is field is collection
     */
    public boolean isCollection() {
        return upperBound > 1;
    }

    /**
     * Return string definition for this field.
     * 
     * @param name field name used for definition
     * @return field definition
     */
    public String toDefinition(String name) {
        String result = type + " " + name;
        if (lowerBound != 0 || upperBound != 0) {
            if (upperBound != NOLIMIT) {
                result += "[" + lowerBound + "-" + upperBound + "]";
            } else {
                result += "[" + lowerBound + "-*]";
            }
        }
        result += WikittyUtil.tagValuesToString(tagValues);
        return result;
    }

    /**
     * Convert value in argument in right type for this FieldType. Don't support
     * collection.
     *
     * @param value value to convert
     * @return object in type of this FieldType
     */
    public Object getContainedValidObject( Object value ) {
        Object result = null;
        switch (type) {
        case BINARY:
            result = WikittyUtil.toBinary(value); break;
        case DATE:
            result = WikittyUtil.toDate(value); break;
        case NUMERIC:
            result = WikittyUtil.toBigDecimal(value); break;
        case BOOLEAN:
            result = WikittyUtil.toBoolean(value); break;
        case STRING:
            result = WikittyUtil.toString(value); break;
        default:
            // if type is not found then type is business type
            // and is wikitty object
            result = WikittyUtil.toWikitty(value); break;
        }
        return result;
    }

    /**
     * Return a valid value for this field.
     * 
     * @param value is casted if possible to an actual correct value.
     * @return value validity
     * @throws WikittyException if value can't be obtained
     */
    public Object getValidValue(Object value) throws WikittyException {
        if (value == null && isNotNull()) {
            throw new WikittyException("Value can't be null for this field");
        }

        Object result;
        if (value == null) {
            result = null;
        } else if (isCollection()) {
            if ( !(value instanceof Collection) ) {
                throw new WikittyException( "A collection is expected for type "
                        + type.name() + "[" + lowerBound + " - " + upperBound + "]" );
            }
            Collection<Object> col;
            if (isUnique()) {
                col = new LinkedHashSet<Object>();
            } else {
                col = new ArrayList<Object>();
            }

            // copy all value in new collections
            for ( Object o : (Collection<?>) value ) {
                col.add( getContainedValidObject(o) );
            }
            result = col;

            // check bound condition
            if (!(getLowerBound() <= col.size() && col.size() <= getUpperBound())) {
                // if upper bound reached, throw an exception
                throw new WikittyException(String.format(
                        "Can't set collection value, bad size collection: %s <= %s <= %s is false",
                        getLowerBound(), col.size(), getUpperBound()));
            }
        } else {
            result = getContainedValidObject(value);
        }
        return result;
    }

    /**
     * Test if value in argument is valid for this field type.
     * 
     * @param value to test
     * @return true if value is valid
     */
    public boolean isValidValue(Object value) {
        return getValidValue(value) != null;
    }

    public WikittyTypes getType() {
        return type;
    }

    public void setType(WikittyTypes type) {
        this.type = type;
    }

    public int getLowerBound() {
        return lowerBound;
    }

    public void setLowerBound(int lowerBound) {
        this.lowerBound = lowerBound;
    }

    public int getUpperBound() {
        return upperBound;
    }

    public void setUpperBound(int upperBound) {
        this.upperBound = upperBound;
    }

    public boolean isUnique() {
        return "true".equalsIgnoreCase(getTagValue(UNIQUE));
    }

    public boolean isNotNull() {
        return "true".equalsIgnoreCase(getTagValue(NOT_NULL));
    }

}
